#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include "hivex.h"


int main(int argc, char **argv) 
{ 

	int flag = HIVEX_OPEN_WRITE;
	if (argc > 1)
	{
		if (argv[1][0]=='d')
		{
			flag |= HIVEX_OPEN_DEBUG;
		}
		else if (argv[1][0]=='v')
		{
			flag |= HIVEX_OPEN_VERBOSE;
		}
	}

	hive_h  *hive_ptr = hivex_open("./system.hive", flag );
	if (NULL == hive_ptr)
	{
		printf("FAILED: hivex_open, error=%d\n", errno);
		return 1;
	}

	hive_node_h root_node_handle = hivex_root(hive_ptr);
	if (0 == root_node_handle)
	{
		printf("FAILED: hivex_root, error=%d\n", errno);
		goto Error;
	}

	const char* currentControlSetName = "ControlSet001";
	hive_node_h currentControlSet = hivex_node_get_child(hive_ptr, root_node_handle, currentControlSetName); 
	if (0 == currentControlSet)
	{
		printf("FAILED: hivex_node_get_child '%s', error=%d\n", currentControlSetName, errno);
		goto Error;
	}

	const char* servicesName = "Services";
	hive_node_h services = hivex_node_get_child(hive_ptr, currentControlSet, servicesName); 
	if ( 0 == services )
	{
		printf("FAILED: hivex_node_get_child '%s', error=%d\n", servicesName, errno);
		goto Error;
	}

	//=========================================================================================
	
	// Case 1: WORKING
	//  1) Hivex library returns handle successfully.
	//  2) CAN see the subkey when hive is mounted in Windows regedit.
	{
		const char* newSubkeyName = "abc";
		hive_node_h newSubkey = hivex_node_add_child(hive_ptr, services, newSubkeyName); 
		if ( 0 == newSubkey )
		{
			printf("hivex_node_add_child %s, error=%d\n", newSubkeyName, errno);
			goto Error;
		}
		else
		{
			printf("successfully added subkey '%s'\n", newSubkeyName);
		}
	}

	// Case 2: FAIL
	//  1) Hivex library returns handle successfully.
	//  2) CANNOT see the subkey when hive is mounted in Windows regedit.
	//     Note that there is another key with name 'symmpi'
	{
		const char* newSubkeyName = "symmpi2";
		hive_node_h newSubkey = hivex_node_add_child(hive_ptr, services, newSubkeyName); 
		if ( 0 == newSubkey )
		{
			printf("hivex_node_add_child %s, error=%d\n", newSubkeyName, errno);
			goto Error;
		}
		else
		{
			printf("successfully added subkey %s\n", newSubkeyName);
		}
	}
	
	// Case 3: FAIL  (Commented out to allow case 1 & 2 runs properly)
	//  1) Hivex library returns handle successfully.
	//  2) CAN see the subkey when hive is mounted in Windows regedit.
	//     However, lots of other subkeys in the same parent disappeared!
	// {
	// const char* newSubkeyName = "_test123_";
	// hive_node_h newSubkey = hivex_node_add_child(hive_ptr, services, newSubkeyName); 
	// if ( 0 == newSubkey )
	// {
	// printf("hivex_node_add_child %s, error=%d\n", newSubkeyName, errno);
	// goto Error;
	// }
	// else
	// {
	// printf("successfully added subkey %s\n", newSubkeyName);
	// }
	// }

	//=========================================================================================

	int result = hivex_commit(hive_ptr, NULL, 0);
	if ( 0 != result )
	{
		printf("FAILED: hivex_commit. result=%d, error=%d\n", result, errno );
		goto Error;
	}

	result = hivex_close(hive_ptr);
	if ( 0 != result )
	{
		printf("FAILED: hivex_close. result=%d, error=%d\n", result, errno );
		return 1;
	}
	
	printf("=== SUCCESS ===\n");
	return 0;


Error:
	result = hivex_close(hive_ptr);
	if ( 0 != result )
	{
		printf("FAILED: hivex_close. result=%d, error=%d\n", result, errno );
		return 1;
	}
	printf("=== ERROR ===\n");
	return 1;
	
} 
